'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _gaussian = require('gaussian');

var _gaussian2 = _interopRequireDefault(_gaussian);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ThreePoint = function () {
  function ThreePoint(items, confidenceLevel) {
    _classCallCheck(this, ThreePoint);

    this.items = items;
    this.confidenceLevel = confidenceLevel || 0.9;
    this.estimate();
  }

  _createClass(ThreePoint, [{
    key: 'findCriticalValue',
    value: function () {
      function findCriticalValue(value) {
        var distribution = (0, _gaussian2['default'])(0, 1);
        return distribution.ppf((1 + value) / 2);
      }

      return findCriticalValue;
    }()
  }, {
    key: 'estimate',
    value: function () {
      function estimate() {
        var estimates = this._items.map(this.estimateItem);

        var standardDeviation = Math.sqrt(estimates.map(function (x) {
          return Math.pow(x.standardDeviation, 2);
        }).reduce(function (a, b) {
          return a + b;
        }));

        var expected = estimates.map(function (x) {
          return x.expected;
        }).reduce(function (a, b) {
          return a + b;
        });

        this._expected = expected;
        this._pessimistic = expected + standardDeviation * this._criticalValue;
        this._optimistic = expected - standardDeviation * this._criticalValue;
      }

      return estimate;
    }()
  }, {
    key: 'estimateItem',
    value: function () {
      function estimateItem(_ref) {
        var optimistic = _ref.optimistic;
        var pessimistic = _ref.pessimistic;
        var likely = _ref.likely;

        return {
          standardDeviation: (pessimistic - optimistic) / 6,
          expected: (optimistic + 4 * likely + pessimistic) / 6
        };
      }

      return estimateItem;
    }()
  }, {
    key: 'items',
    set: function () {
      function set(value) {
        value.forEach(function (_ref2) {
          var optimistic = _ref2.optimistic;
          var pessimistic = _ref2.pessimistic;
          var likely = _ref2.likely;

          if (isNaN(optimistic) || isNaN(pessimistic) || isNaN(likely)) {
            throw new Error('Encountered a non-number in estimate items');
          }
        });

        this._items = value;
        this.estimate();
      }

      return set;
    }()
  }, {
    key: 'confidenceLevel',
    set: function () {
      function set(value) {
        this._criticalValue = this.findCriticalValue(value);
        this.estimate();
      }

      return set;
    }()
  }, {
    key: 'expected',
    get: function () {
      function get() {
        return Math.round(this._expected * 10) / 10;
      }

      return get;
    }()
  }, {
    key: 'pessimistic',
    get: function () {
      function get() {
        return Math.round(this._pessimistic * 10) / 10;
      }

      return get;
    }()
  }, {
    key: 'optimistic',
    get: function () {
      function get() {
        return Math.round(this._optimistic * 10) / 10;
      }

      return get;
    }()
  }]);

  return ThreePoint;
}();

exports['default'] = ThreePoint;
module.exports = exports['default'];
